#ifndef META_HPP__
#define META_HPP__

#include "common/data.hpp"
#include <opencv2/opencv.hpp>
#include <vector>
#include <string>

namespace meta
{
// 数据类型
enum class DataType {
    FRAME = 0, // 帧数据
    CONFIG = 2 // 配置数据
};

class BaseData
{
public:
    BaseData() = delete;
    explicit BaseData(DataType data_type) {
        this->data_type = data_type;
        create_time     = std::chrono::system_clock::now();
    }

    virtual ~BaseData() {}

public:
    DataType                              data_type;    // 数据类型
    std::chrono::system_clock::time_point create_time;  // 数据创建时间
    std::string                           data_name;    // 数据名称/数据来源

};


class FrameData : public BaseData
{
public:
    explicit FrameData() : BaseData(DataType::FRAME) {
        data_name = "FrameData";
    }

public:
    std::string from; // 数据来源
    int fps = 20;   // 帧率
    cv::Mat image; // 原始图片
    cv::Mat draw_image; // 画框图
    cv::Mat depth; // 深度图
    data::BoxArray boxes; // 目标检测识别结果
    data::BoxArray track_boxes; // 跟踪结果
    data::BoxArray result; // 分析结果
};


class AnalyzeConfigData : public BaseData
{

public:
    explicit AnalyzeConfigData() : BaseData(DataType::CONFIG) {
        data_name = "AnalyzeConfigData";
    }

public:
    std::vector<std::string> algorithm_names; // 算法名称
    std::unordered_map<std::string, std::vector<std::vector<cv::Point>>> algorithm_params_fence; // 算法电子围栏坐标点

};


class DrawConfigData : public BaseData
{

public:
    explicit DrawConfigData() : BaseData(DataType::CONFIG) {
        data_name = "DrawConfigData";
    }

public:
    bool show_final_result = true; // 是否显示最终结果
    bool show_original_result = true; // 是否显示原始结果
};


class SourceConfigData : public BaseData
{
public:
    explicit SourceConfigData() : BaseData(DataType::CONFIG) {
        data_name = "SourceConfigData";
    }

public:
    int gpu_id = 0;
    int skip_frame = 1; // 跳过的帧数


};


}


#endif // META_HPP__