#ifndef INFER_HPP__
#define INFER_HPP__

#include "common/data.hpp"
#include "common/meta.hpp"
#include "common/image.hpp"

#include <variant>

enum class ModelType : int{
    YOLOV5         = 0,
    YOLOV5SEG      = 1,
    YOLO11         = 2,
    YOLO11POSE     = 3,
    YOLO11SEG      = 4,
    DEPTH_ANYTHING = 5
};

using Result = std::variant<data::BoxArray, cv::Mat, std::monostate>;

class Infer{
public:
    virtual Result forward(const tensor::Image &image, int slice_width, int slice_height, float overlap_width_ratio, float overlap_height_ratio, void *stream = nullptr) = 0;
    virtual Result forward(const tensor::Image &image, void *stream = nullptr) = 0;
    virtual Result forward(const tensor::Image &image, data::BoxArray& boxes, void *stream = nullptr) = 0;

    virtual int get_gpu_id() = 0;

    virtual void set_nms_threshold(float nms_threshold)
    {

    }

    virtual void set_conf_threshold(float conf_threshold)
    {

    }

protected:
    std::mutex mutex_;
};

std::shared_ptr<Infer> load(const std::string& model_path, ModelType model_type, const std::vector<std::string>& names, int gpu_id=0, float confidence_threshold=0.5f, float nms_threshold=0.45f);

#endif // INFER_HPP__