#ifndef CLASSIFIER_HPP__
#define CLASSIFIER_HPP__

#include <vector>
#include <iomanip>
#include "common/memory.hpp"
#include "infer/infer.hpp"
#include "common/image.hpp"
#include "common/data.hpp"
#include "infer/trt/affine.hpp"

#include "common/data.hpp"

#ifdef TRT10
#include "common/tensorrt.hpp"
namespace TensorRT = TensorRT10;
#else
#include "common/tensorrt8.hpp"
namespace TensorRT = TensorRT8;
#endif

/**
 * 对目标检测后识别到的框进行二次分类
 */

namespace cls
{

class ClassifierModelImpl : public Infer
{

public:
    std::shared_ptr<TensorRT::Engine> trt_;
    std::vector<std::string> class_names_;
    std::string engine_file_;
    tensor::Memory<float> affine_matrix_;

    tensor::Memory<float> input_buffer_, output_buffer_;
    tensor::Memory<int> classes_indices_;

    tensor::Memory<unsigned char> preprocess_buffer_;

    int num_classes_ = 0;
    int device_id_ = 0;

    int network_input_width_;
    int network_input_height_;

    bool isdynamic_model_ = false;

    affine::Norm normalize_;

public:
    ClassifierModelImpl() = default;
    virtual ~ClassifierModelImpl() = default;

    virtual int get_gpu_id() override
    {
        return device_id_;
    }

    void adjust_memory(int batch_size, int image_width, int image_height) 
    {
        // the inference batch_size
        size_t input_numel = network_input_width_ * network_input_height_ * 3;
        input_buffer_.gpu(batch_size * input_numel);
        output_buffer_.gpu(batch_size * num_classes_);
        classes_indices_.gpu(batch_size);
        classes_indices_.cpu(batch_size);
        preprocess_buffer_.gpu(image_width * image_height * 3);
      }

    bool load(const std::string &engine_file, int gpu_id);

    void preprocess(int ibatch, const tensor::Image &image, affine::CropResizeMatrix& matrix, int x, int y, int w, int h, void *stream);
    
    virtual Result forward(const tensor::Image &image, void *stream = nullptr);
    virtual Result forward(const tensor::Image &image, int slice_width, int slice_height, float overlap_width_ratio, float overlap_height_ratio, void *stream = nullptr);
    virtual Result forward(const tensor::Image &image, data::BoxArray& boxes, void *stream = nullptr);

};

}


#endif