#ifndef DRAWNODE_HPP__
#define DRAWNODE_HPP__
#include "nodes/base/base.hpp"
#include <opencv2/opencv.hpp>
#include <unordered_map>

// 日志库
#include "plog/Log.h"
#include "plog/Initializers/RollingFileInitializer.h"


namespace GNode
{

class DrawNode : public BaseNode
{
public:
    DrawNode() = delete;
    DrawNode(const std::string& name) : BaseNode(name, NODE_TYPE::DES_NODE) {}
    virtual ~DrawNode() { stop(); };

    void handle_data(std::shared_ptr<meta::BaseData>& meta_data) override;

private:
    void draw_boxes(cv::Mat& image, const data::BoxArray& boxes);
    void draw_track_boxes(cv::Mat& image, const data::BoxArray& track_boxes);
    void draw_result(cv::Mat& image, const data::BoxArray& result);

private:
    std::unordered_map<int, std::vector<cv::Point>> track_history_;
    std::unordered_map<int, std::chrono::steady_clock::time_point> last_track_update_time_;
};

} // namespace Node


#endif // DRAWNODE_HPP__