#include "common/utils.hpp"
#include "nodes/base/base.hpp"
#include "nodes/httpPush/httpPush.hpp"

namespace GNode
{

static cv::Mat get_color_depth(cv::Mat inverse_depth)
{
    double min_val, max_val;
    cv::minMaxLoc(inverse_depth, &min_val, &max_val);

    float max_invdepth_vizu = std::min(static_cast<float>(max_val), 1.0f / 0.1f);
    float min_invdepth_vizu = std::max(static_cast<float>(1.0 / 250.0), static_cast<float>(min_val));

    cv::Mat inverse_depth_normalized = (inverse_depth - min_invdepth_vizu) / (max_invdepth_vizu - min_invdepth_vizu);
    
    cv::Mat inverse_depth_normalized_8u;
    inverse_depth_normalized.convertTo(inverse_depth_normalized_8u, CV_8UC1, 255);

    cv::Mat color_depth;
    cv::applyColorMap(inverse_depth_normalized_8u, color_depth, cv::COLORMAP_JET);

    cv::Mat color_depth_bgr;
    cv::cvtColor(color_depth, color_depth_bgr, cv::COLOR_BGR2RGB);
    return color_depth_bgr;
}

void HttpPushNode::handle_data(std::shared_ptr<meta::BaseData>& meta_data)
{
    auto frame_data = std::dynamic_pointer_cast<meta::FrameData>(meta_data);
    auto result = frame_data->result;

    if (result.size() == 0)
    {
        return;
    }

    cv::Mat image = frame_data->draw_image;

    std::string image_name = "result/" + frame_data->from + "_" + getTimeString() + ".jpg";
    cv::imwrite(image_name, image);

    if (!frame_data->depth.empty())
    {
        std::string depth_name = "result/" + frame_data->from + "_depth_" + getTimeString() + ".jpg";
        cv::imwrite(depth_name, get_color_depth(frame_data->depth));
    }
}

}   // namespace Node