#include "nodes/base/base.hpp"
#include "nodes/infer/inferNode.hpp"
#include "common/image.hpp"
#include "common/utils.hpp"
#include "common/check.hpp"
#include <unordered_map>
#include <random>
#include <algorithm>

namespace GNode
{


void print_mat(const cv::Mat& mat, int max_rows = 10, int max_cols = 10)
{
    for (int i = 0; i < std::min(max_rows, mat.rows); i++) {
        for (int j = 0; j < std::min(max_cols, mat.cols); j++) {
            std::cout << mat.at<float>(i,j) << " ";
        }
        std::cout << (mat.cols > max_cols ? "..." : "") << std::endl;
    }
    if (mat.rows > max_rows) std::cout << "[...]" << std::endl;
}


void InferNode::handle_data(std::shared_ptr<meta::BaseData>& meta_data)
{
    auto frame_data = std::dynamic_pointer_cast<meta::FrameData>(meta_data);
    checkRuntime(cudaSetDevice(device_id_));
    cv::Mat image = frame_data->image;
    int width = image.cols;
    int height = image.rows;

    auto det_result = model_->forward(tensor::cvimg(image), image.cols, image.rows, 0.0f, 0.0f);

    if (std::holds_alternative<data::BoxArray>(det_result)) 
    {
        auto result = std::get<data::BoxArray>(det_result);
        for (auto& r : result)
        {
            frame_data->boxes.push_back(r);
        }
    } 
    else if(std::holds_alternative<cv::Mat>(det_result))
    {
        auto depth_mat = std::get<cv::Mat>(det_result);
        // print_mat(depth_mat);
        frame_data->depth = depth_mat;

    }
    else
    {
        PLOGE.printf("InferNode : [%s] Unexpected result type from model", name_.c_str());
        throw std::runtime_error("Unexpected result type from model");
    }
}

}   // namespace StreamNode