#ifndef TRACKNODE_HPP__
#define TRACKNODE_HPP__

#include "nodes/base/base.hpp"
#include <opencv2/opencv.hpp>
#include "common/meta.hpp"
#include "common/data.hpp"
#include "BYTETracker.h"
#include "infer/infer.hpp"

// 日志库
#include "plog/Log.h"
#include "plog/Initializers/RollingFileInitializer.h"
namespace GNode
{

class TrackNode : public BaseNode
{
public:
    TrackNode() = delete;
    TrackNode(const std::string& name, const std::string& track_label, int frame_rate=30, int track_buffer=30) : BaseNode(name, NODE_TYPE::MID_NODE) 
    {
        track_label_ = track_label;
        frame_rate_ = frame_rate;
        track_buffer_ = track_buffer;
        PLOGI.printf("TrackNode : [%s] Init. track label is %s, rate is %d, buffer is %d", name_.c_str(), track_label_.c_str(), frame_rate_, track_buffer_);
    }
    virtual ~TrackNode()  { stop(); };
    void handle_data(std::shared_ptr<meta::BaseData>& meta_data) override;

private:
    std::unordered_map<std::string, std::shared_ptr<BYTETracker>> track_map_;
    std::string track_label_;

    int frame_rate_;
    int track_buffer_;

};

}

#endif  // TRACKNODE_HPP__